package api

import (
	"ArmedPolice/app/controller/role"
	"ArmedPolice/app/service"
	"github.com/gin-gonic/gin"
)

type Role struct{}

type roleForm struct {
	ParentID string `json:"parent_id" form:"parent_id"`
	Name     string `json:"name" form:"name" binding:"required"`
	Remark   string `json:"remark" form:"remark" binding:"required"`
	Sort     int    `json:"sort" form:"sort"`
}

func (this *roleForm) parentInfo() uint64 {
	obj := new(IDStringForm)
	obj.ID = this.ParentID
	return obj.Convert()
}

/**
 * @apiDefine Role 角色管理
 */

/**
 * @api {get} /api/v1/role/list 角色列表
 * @apiVersion 1.0.0
 * @apiName RoleList
 * @apiGroup Role
 *
 * @apiHeader {string} x-token token
 *
 * @apiSuccess (200) {Number} code 成功响应状态码!
 * @apiSuccess (200) {String} msg 成功提示
 * @apiSuccess (200) {Array} data 具体信息
 * @apiSuccess (200) {String} data.id 角色ID
 * @apiSuccess (200) {String} data.name 角色名称
 * @apiSuccess (200) {Array} data.children="[]" 子集
 *
 * @apiSuccessExample {json} Success response:
 *     HTTPS 200 OK
 *     {
 *      "code": 200
 *      "msg": "ok"
 *		"data": [
 *			{
 *				"id": "EgmJ4Ga7LQ",
 *				"name": "分管领导",
 *				"children": [
 *					{
 *						"id": "7K1Jx6VYod",
 *                  	"name": "区域支队",
 *					}
 *				]
 *			}
 *		]
 *    }
 */
func (*Role) List(c *gin.Context) {
	data, err := role.NewInstance()(getSession()(c).(*service.Session)).List()
	APIResponse(err, data)(c)
}

func (a *Role) Add(c *gin.Context) {
	form := new(roleForm)

	if err := bind(form)(c); err != nil {
		APIFailure(err.(error))(c)
		return
	}
	err := role.NewInstance()(getSession()(c).(*service.Session)).Form(&role.InstanceParams{
		ParentID: form.parentInfo(), Name: form.Name, Remark: form.Remark, Sort: form.Sort,
	})
	APIResponse(err)(c)
}

func (a *Role) Edit(c *gin.Context) {
	form := &struct {
		IDStringForm
		roleForm
	}{}
	if err := bind(form)(c); err != nil {
		APIFailure(err.(error))(c)
		return
	}
	err := role.NewInstance()(getSession()(c).(*service.Session)).Form(&role.InstanceParams{
		ID: form.Convert(), ParentID: form.parentInfo(), Name: form.Name, Remark: form.Remark, Sort: form.Sort,
	})
	APIResponse(err)(c)
}

func (a *Role) Delete(c *gin.Context) {
	form := new(IDStringForm)

	if err := bind(form)(c); err != nil {
		APIFailure(err.(error))(c)
		return
	}
	err := role.NewInstance()(getSession()(c).(*service.Session)).Delete(form.Convert())
	APIResponse(err)(c)
}

/**
 * @api {post} /api/v1/role/menus 角色菜单
 * @apiVersion 1.0.0
 * @apiName RoleMenu
 * @apiGroup Role
 *
 * @apiHeader {string} x-token token
 *
 * @apiParam {Number} role_id 角色ID
 *
 * @apiSuccess (200) {Number} code 成功响应状态码!
 * @apiSuccess (200) {String} msg 成功提示
 * @apiSuccess (200) {Array} data 具体信息
 * @apiSuccess (200) {String} data.id 菜单ID
 * @apiSuccess (200) {Number} data.parent_id 父级ID
 * @apiSuccess (200) {String} data.name 菜单名称
 * @apiSuccess (200) {Number} data.kind 类型（1：目录，2：菜单）
 * @apiSuccess (200) {String} data.link 访问地址
 * @apiSuccess (200) {String} data.component 组件
 * @apiSuccess (200) {Array} data.children="[]" 子集
 *
 * @apiSuccessExample {json} Success response:
 *     HTTPS 200 OK
 *     {
 *      "code": 200
 *      "msg": "ok"
 *		"data": [
 *			"id": 1,
 *			"parent_id": 0,
 *			"name": "系统管理",
 *			"kind": 1,
 *			"link": "1"
 *			"component": ""
 *			"children": [
 *				{
 *					"id": 2,
 *					"parent_id": 1,
 *                  "name": "用户管理",
 *                  "kind": 1,
 *                  "link": "1"
 *                  "component": ""
 *                  "children": [],
 *				}
 *			]
 *		]
 *    }
 */
func (a *Role) Menu(c *gin.Context) {
	form := &struct {
		RoleID string `json:"role_id" form:"role_id" binding:"required"`
	}{}
	if err := bind(form)(c); err != nil {
		APIFailure(err.(error))(c)
		return
	}
	obj := new(IDStringForm)
	obj.ID = form.RoleID
	data, err := role.NewMenu()(getSession()(c).(*service.Session)).List(obj.Convert())
	APIResponse(err, data)(c)
}

/**
 * @api {post} /api/v1/role/menu/bind 角色菜单绑定
 * @apiVersion 1.0.0
 * @apiName RoleMenuBind
 * @apiGroup Role
 *
 * @apiHeader {string} x-token token
 *
 * @apiParam {Number} role_id 角色ID
 * @apiParam {Array.String} menu_ids 菜单ID
 *
 * @apiSuccess (200) {Number} code=200 成功响应状态码!
 * @apiSuccess (200) {String} msg="ok" 成功提示
 *
 * @apiSuccessExample {json} Success response:
 *     HTTPS 200 OK
 *     {
 *     	"code": 200
 *		"msg": "ok"
 *		"data": null
 *     }
 */
func (a *Role) MenuBind(c *gin.Context) {
	form := &struct {
		RoleID  string   `json:"role_id" form:"role_id" binding:"required"`
		MenuIDs []string `json:"menu_ids" form:"menu_ids" binding:"required"`
	}{}
	if err := bind(form)(c); err != nil {
		APIFailure(err.(error))(c)
		return
	}
	handle := new(IDStringForm)
	handle.ID = form.RoleID
	roleID := handle.Convert()
	menuIDs := make([]uint64, 0)

	for _, v := range form.MenuIDs {
		handle.ID = v
		menuIDs = append(menuIDs, handle.Convert())
	}
	err := role.NewMenu()(getSession()(c).(*service.Session)).Bind(roleID, menuIDs)
	APIResponse(err)(c)
}
